classdef TransformedSystem < FunctionSystem
    
    properties(SetAccess = private)
        fhandle
        Ap
        bp
    end
    
    methods
        function obj = TransformedSystem(f, A, b)
           if ~isa(f, 'function_handle')
              error('The argument f must be a function handle.'); 
           end
           if ~isvector(A) || ~isvector(b) || (length(A) ~= length(b))
              error('The arguments A and b must be vectors of the same size.'); 
           end
           
           obj.N = length(A);
           
           obj.fhandle = f;
           obj.Ap = reshape(A, obj.N, 1);
           obj.bp = reshape(b, obj.N, 1);
        end
        
        function y = evaluate(obj, x)
            if ~isscalar(x)
               error('The argument x must be a scalar value.'); 
            end
           
            x_transformed = obj.Ap.*x + obj.bp;
            detA = abs(obj.Ap);
            
            %TODO: Is f isn't a function that doesn't operate elementwise
            %on a vector, replace by arrayfun(works in every case, but is
            %slower).
            y = f(x_transformed).*detA;
        end
        
        function y = evaluateAll(obj, x)
            if ~isvector(x)
              error('The argument x must be a vector.'); 
            end
            
            evlpoints = repmat(reshape(x, 1, length(x)), obj.N, 1);
            
            Arep = repmat(obj.Ap, 1, length(x));
            brep = repmat(obj.bp, 1, length(x));
            
            %TODO: If necessary, replace by arrayfun
           y = obj.fhandle(Arep.*evlpoints + brep).*abs(Arep);
        end
        
        function F = get(obj, i)
            if ~isscalar(i) || (i < 1) || (i > obj.N)
               error('The argument i is not a suitable index.'); 
            end
            
            A = obj.Ap(i);
            b = obj.bp(i);
            
            g = @(x) obj.fhandle(A.*x + b).*abs(A);
            F = HandleFunction(g);
        end
    end
    
end

