classdef Triangulation < handle
    
    properties(SetAccess = protected)
        points
        edges
    end
    
    methods
        
        function obj = Triangulation(p, e)
            if ~isvector(p)
               error('The first argument must be a vector.'); 
            end
            if size(e, 2) == 2
                if ~all((e(:) > 0) & (e(:) <= length(p)))
                    error('The argument e is not suitable.');
                end
            else
               error('The argument p must be a n-by-2-matrix.'); 
            end
            
            obj.points = reshape(p, length(p), 1);
            obj.edges = e;
        end
        
        
        function F = getSystem(obj, f)
           if ~isa(f, 'function_handle')
              error('The parameter f must be a function handle.');
           end
           
           bp = obj.points(obj.edges(:, 1));
           Ap = obj.points(obj.edges(:,2)) - obj.points(obj.edges(:, 1));
           
           F = TransformedSystem(f, Ap, bp);
        end
    end
    
end

